ConcurrentAllocator

Trait ConcurrentAllocator 

pub trait ConcurrentAllocator:
    ApiAllocator
    + Clone
    + Send
    + Sync
    + 'static { }
Allocator safe to clone into concurrent data structures and deferred drops.

Cloning must be cheap and must not panic. The 'static bound allows deferred reclamation to outlive the container that captured the allocator.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl<A: ApiAllocator + Clone + Send + Sync + 'static> ConcurrentAllocator for A