pub enum Register {
Value(Value),
Aggregate(AggContext),
Record(ImmutableRecord),
}Variants
Implementations
Sourceimpl Register
impl Register
pub const fn is_null(&self) -> bool
Sourcepub fn set_int(&mut self, val: i64)
pub fn set_int(&mut self, val: i64)
Sets the value of the register to an integer, reusing the existing Register::Value(Value::Numeric(Numeric::Integer(_))) if possible, which is faster than always creating a new one.
Sourcepub fn set_float(&mut self, val: NonNan)
pub fn set_float(&mut self, val: NonNan)
Set the value of the register to a floating point, reusing Register::Value(Value::Numeric(Numeric::Float(_))) if possible.
Sourcepub fn set_text(&mut self, val: Text) -> Result<()>
pub fn set_text(&mut self, val: Text) -> Result<()>
Set the value of the register to a Text, reusing Register::Value(Value::Text(_)) buffer if possible.
Sourcepub fn set_blob(&mut self, val: Vec<u8>) -> Result<()>
pub fn set_blob(&mut self, val: Vec<u8>) -> Result<()>
Set the value of the register to a blob, reusing Register::Value(Value::Blob(_)) buffer if possible.
pub fn set_null(&mut self)