Module index_method

Module index_method 

Modules

backing_btree
toy_vector_sparse_ivf

Structs

IndexMethodConfiguration
IndexMethodCostEstimate
Cost estimate returned by custom index methods for optimizer integration. This enables the optimizer to make cost-based decisions when choosing between custom index methods and traditional BTree indexes.
IndexMethodDefinition

Constants

BACKING_BTREE_INDEX_METHOD_NAME
TOY_VECTOR_SPARSE_IVF_INDEX_METHOD_NAME

Traits

IndexMethod
index method “entry point” which can create attachment of the method to the table with given configuration (this trait acts like a “factory”)
IndexMethodAttachment
index method attached to the table with specific configuration the attachment is capable of generating SELECT patterns where index can be used and also can create cursor for query execution
IndexMethodCursor
cursor opened for index method and capable of executing DML/DDL/DQL queries for the index method over fixed table