pub struct BackingBtreeIndexMethod;Expand description
Special ‘backing_btree’ index method which can be used by other custom index methods
Under the hood, it’s marked as ‘treat_as_btree’ which recognized by the tursodb core as a special index method which should be translated to ordinary btree but also do not explicitly managed by the core