pub struct VectorSparseInvertedIndexMethod;Expand description
Simple inverted index for sparse vectors
CREATE INDEX t_idx ON t USING toy_vector_sparse_ivf (embedding)
It accept single column which must contain vector encoded in sparse format (e.g. vector32_sparse(…)) It can handle jaccard similarity scoring queries like the following:
SELECT vector_distance_jaccard(embedding, ?) as d FROM t ORDER BY d LIMIT ?