pub enum FileSyncType {
Fsync,
FullFsync,
}Expand description
Controls which sync mechanism to use for durability.
FullFsync only has effect on Apple platforms (uses F_FULLFSYNC fcntl).
On other platforms, both variants behave the same (regular fsync).
Variants
Fsync
Regular fsync - flushes to disk but may not flush disk write cache on macOS.
FullFsync
Full fsync - on macOS uses F_FULLFSYNC to flush disk write cache. On other platforms, behaves the same as Fsync.