macro_rules! get_cursor {
($state:expr, $cursor_id:expr) => { ... };
}Expand description
Macro is necessary to help the borrow checker see we are only accessing state.cursor field and nothing else
macro_rules! get_cursor {
($state:expr, $cursor_id:expr) => { ... };
}Macro is necessary to help the borrow checker see we are only accessing state.cursor field and nothing else