pub enum NextRowidResult {
Uninitialized,
Next {
new_rowid: i64,
prev_rowid: Option<i64>,
},
FindRandom,
}Variants
Uninitialized
We need to go to the last rowid and intialize allocator
Next
It was initialized, so we get a new rowid
FindRandom
We reached end of available rowids (i64::MAX), so we will have to try and find a random rowid.