pub struct LockStates { /* private fields */ }Expand description
The states of the locks held by the state machine - these are tracked for error handling so that they are released if the state machine fails.