pub enum WriteRowState {
Initial,
Seek,
Advance,
Insert,
Next,
}Variants
Initial
Seek
Advance
After seek returns TryAdvance for an index key stored in an interior node, advance the cursor to that interior cell so insert overwrites it.
Insert
Next
Move to the next record in order to leave the cursor in the next position, this is used for inserting multiple rows for optimizations.