MVTableId

Struct MVTableId 

pub struct MVTableId(/* private fields */);
A table ID for MVCC. MVCC table IDs are always negative. Their corresponding rootpage entry in sqlite_schema is the same negative value if the table has not been checkpointed yet. Otherwise, the root page will be positive and corresponds to the actual physical page.

Implementations

Source

impl MVTableId

pub fn new(value: i64) -> Self