pub struct RowVersion {
pub id: u64,
pub row: Row,
pub btree_resident: bool,
/* private fields */
}Expand description
A row version.
Fields
id: u64Unique identifier for this version within the MvStore. Used for savepoint tracking to identify specific versions to rollback.
row: Rowbtree_resident: boolIndicates this version was created for a row that existed in B-tree before MVCC was enabled (e.g., after switching from WAL to MVCC journal mode). This flag helps the checkpoint logic determine if a delete should be checkpointed to the B-tree file.
Implementations
Sourceimpl RowVersion
impl RowVersion
Sourcepub fn new(
id: u64,
begin: Option<TxTimestampOrID>,
end: Option<TxTimestampOrID>,
row: Row,
btree_resident: bool,
) -> Self
pub fn new( id: u64, begin: Option<TxTimestampOrID>, end: Option<TxTimestampOrID>, row: Row, btree_resident: bool, ) -> Self
Construct a row version. begin/end are bit-packed internally.
Sourcepub fn begin(&self) -> Option<TxTimestampOrID>
pub fn begin(&self) -> Option<TxTimestampOrID>
The begin timestamp/tx-id, or None.
Sourcepub fn end(&self) -> Option<TxTimestampOrID>
pub fn end(&self) -> Option<TxTimestampOrID>
The end timestamp/tx-id, or None.
pub fn set_begin(&mut self, value: Option<TxTimestampOrID>)
pub fn set_end(&mut self, value: Option<TxTimestampOrID>)