pub enum ColumnLayout {
Identity {
column_count: usize,
},
Mapped {
offsets: Vec<usize>,
non_virtual_col_count: usize,
},
}Expand description
Maps schema column indices to register offsets for DML operations.
Variants
Implementations
Sourceimpl ColumnLayout
impl ColumnLayout
pub fn from_table(table: &Table) -> Result<Self, TryReserveError>
pub fn from_btree(btree: &BTreeTable) -> Result<Self, TryReserveError>
pub fn from_columns(columns: &[Column]) -> Result<Self, TryReserveError>
Sourcepub fn to_reg_offset(&self, col_idx: usize) -> usize
pub fn to_reg_offset(&self, col_idx: usize) -> usize
Map a schema column index to its register offset.
Sourcepub fn to_register(&self, base: usize, schema_idx: usize) -> usize
pub fn to_register(&self, base: usize, schema_idx: usize) -> usize
Resolve schema column index to an absolute register.
pub fn num_non_virtual_cols(&self) -> usize
pub fn column_count(&self) -> usize
pub fn column_idx_for_offset(&self, offset: usize) -> Option<usize>