pub struct Column {
pub name: Option<String>,
pub ty_str: String,
pub ty_params: Vec<Box<Expr>>,
pub default: Option<Box<Expr>>,
pub notnull_conflict_clause: Option<ResolveType>,
/* private fields */
}Fields
name: Option<String>ty_str: Stringty_params: Vec<Box<Expr>>default: Option<Box<Expr>>notnull_conflict_clause: Option<ResolveType>ON CONFLICT clause for NOT NULL constraint on this column.
Implementations
Sourceimpl Column
impl Column
pub fn affinity(&self) -> Affinity
Sourcepub fn set_base_affinity(&mut self, affinity: Affinity)
pub fn set_base_affinity(&mut self, affinity: Affinity)
Set the base type affinity override for a custom type column. This ensures affinity rules use the custom type’s BASE type rather than applying SQLite name-based rules to the type name.
pub fn affinity_with_strict(&self, is_strict: bool) -> Affinity
pub fn new_default_text( name: Option<String>, ty_str: String, default: Option<Box<Expr>>, ) -> Self
pub fn new_default_integer( name: Option<String>, ty_str: String, default: Option<Box<Expr>>, ) -> Self
pub fn new( name: Option<String>, ty_str: String, default: Option<Box<Expr>>, generated: Option<Box<Expr>>, ty: Type, col: Option<CollationSeq>, coldef: ColDef, ) -> Self
pub const fn ty(&self) -> Type
pub const fn set_ty(&mut self, ty: Type)
pub const fn collation_opt(&self) -> Option<CollationSeq>
pub const fn collation(&self) -> CollationSeq
pub const fn has_explicit_collation(&self) -> bool
pub const fn set_collation(&mut self, c: Option<CollationSeq>)
pub fn primary_key(&self) -> bool
pub const fn is_rowid_alias(&self) -> bool
pub const fn notnull(&self) -> bool
pub const fn explicit_notnull(&self) -> bool
pub const fn unique(&self) -> bool
Sourcepub fn ensure_not_generated(
&self,
verb_phrase: &str,
col_name: &str,
) -> Result<()>
pub fn ensure_not_generated( &self, verb_phrase: &str, col_name: &str, ) -> Result<()>
Returns an error if this column is a generated column.
verb_phrase should describe the operation, e.g. “INSERT into” or “UPDATE”.
pub fn generated_type(&self) -> &GeneratedType
pub const fn is_generated(&self) -> bool
pub const fn is_virtual_generated(&self) -> bool
pub fn generated_expr(&self) -> Option<&Expr>
pub fn generated_expr_mut(&mut self) -> Option<&mut Expr>
pub fn set_generated_original_sql(&mut self, new_sql: String)
pub const fn set_primary_key(&mut self, v: bool)
pub const fn set_rowid_alias(&mut self, v: bool)
pub const fn set_notnull(&mut self, v: bool)
pub const fn set_unique(&mut self, v: bool)
pub const fn is_array(&self) -> bool
Sourcepub const fn array_dimensions(&self) -> u32
pub const fn array_dimensions(&self) -> u32
Number of array dimensions (0 = scalar, 1 = [], 2 = [][], etc.)
pub fn set_array_dimensions(&mut self, dims: u32)