pub struct FromClauseSubqueryCteMetadata {
pub id: usize,
pub shared_materialization: bool,
pub materialize_hint: bool,
}Fields
id: usizeIdentity shared by all references to the same CTE definition.
True when more than one read in the same query tree can reuse one materialized result for this CTE.
materialize_hint: boolTrue for explicit WITH … AS MATERIALIZED.