pub struct Entry<'a: 'g, 'g, K, V, C = BasicComparator, A: SkiplistAllocator = TursoAllocator> { /* private fields */ }Expand description
An entry in a skip list, protected by a Guard.
The lifetimes of the key and value are the same as that of the Guard
used when creating the Entry ('g). This lifetime is also constrained to
not outlive the SkipList.
Implementations
Sourceimpl<'a: 'g, 'g, K: 'a, V: 'a, C, A: SkiplistAllocator> Entry<'a, 'g, K, V, C, A>
impl<'a: 'g, 'g, K: 'a, V: 'a, C, A: SkiplistAllocator> Entry<'a, 'g, K, V, C, A>
Sourcepub fn is_removed(&self) -> bool
pub fn is_removed(&self) -> bool
Returns true if the entry is removed from the skip list.
Sourceimpl<K, V, C, A: SkiplistAllocator> Entry<'_, '_, K, V, C, A>
impl<K, V, C, A: SkiplistAllocator> Entry<'_, '_, K, V, C, A>
Sourceimpl<K, V, C, A: SkiplistAllocator> Entry<'_, '_, K, V, C, A>where
C: Comparator<K>,
impl<K, V, C, A: SkiplistAllocator> Entry<'_, '_, K, V, C, A>where
C: Comparator<K>,