RefEntry

Struct RefEntry 

pub struct RefEntry<'a, K, V, C = BasicComparator, A: SkiplistAllocator = TursoAllocator> { /* private fields */ }
A reference-counted entry in a skip list.

You must call release to free this type, otherwise the node will be leaked. This is because releasing the entry requires a Guard.

Implementations

Source

impl<'a, K: 'a, V: 'a, C: 'a, A: SkiplistAllocator> RefEntry<'a, K, V, C, A>

pub fn is_removed(&self) -> bool

Returns true if the entry is removed from the skip list.

pub fn key(&self) -> &'a K

Returns a reference to the key.

pub fn value(&self) -> &'a V

Returns a reference to the value.

pub fn skiplist(&self) -> &'a SkipList<K, V, C, A>

Returns a reference to the parent SkipList

pub fn release(self, guard: &Guard)

Releases the reference on the entry.

pub fn release_with_pin<F>(self, pin: F)
where F: FnOnce() -> Guard,

Releases the reference of the entry, pinning the thread only when the reference count of the node becomes 0.

Source

impl<K, V, C, A: SkiplistAllocator> RefEntry<'_, K, V, C, A>
where C: Comparator<K>, K: Send + 'static, V: Send + 'static,

pub fn remove(&self, guard: &Guard) -> bool

Removes the entry from the skip list.

Returns true if this call removed the entry and false if it was already removed.

Source

impl<K, V, C, A: SkiplistAllocator> RefEntry<'_, K, V, C, A>
where C: Comparator<K>,

pub fn move_next(&mut self, guard: &Guard) -> bool

Moves to the next entry in the skip list.

pub fn next(&self, guard: &Guard) -> Option<Self>

Returns the next entry in the skip list.

pub fn move_prev(&mut self, guard: &Guard) -> bool

Moves to the previous entry in the skip list.

pub fn prev(&self, guard: &Guard) -> Option<Self>

Returns the previous entry in the skip list.