pub struct BasicComparator;Expand description
This comparator uses the Equivalent and Comparable traits to perform
comparisons, which themselves fall back on the standard library Borrow
and Ord traits. When used in a map, this results in the same behavior as
the standard BTreeMap interface.