pub struct StateMachine<State: StateTransition> { /* private fields */ }Implementations
Sourceimpl<State: StateTransition> StateMachine<State>
A generic state machine that loops calling transition until it returns TransitionResult::Done or TransitionResult::Io.
impl<State: StateTransition> StateMachine<State>
A generic state machine that loops calling transition until it returns TransitionResult::Done or TransitionResult::Io.