WalFile

Struct WalFile 

pub struct WalFile { /* private fields */ }

Implementations

Source

impl WalFile

pub fn new( io: Arc<dyn IO>, shared: Arc<RwLock<WalFileShared>>, (last_checksum, max_frame): ((u32, u32), u64), buffer_pool: Arc<BufferPool>, ) -> Self

pub fn try_restart_log_before_write(&self) -> Result<()>

attempt to restart WAL header before write in order to keep WAL file size under the control The conditions for WAL restart are following:

  1. we can do that only under write transaction
  2. max_frame_read_lock_index == 0 - this means that transaction was initiated to read data from DB file
  3. nbackfills > 0 - otherwise nothing was backfilled and there is no reason to truncate header
  4. max_frame == nbackfills - otherwise there are some non-checkpointed frames in the WAL and we can’t truncate the log
pub fn mvcc_refresh_if_db_changed(&self) -> bool

MVCC helper: check if WAL state changed and refresh local snapshot without starting a read tx. FIXME: this isn’t TOCTOU safe because we’re not taking WAL read locks.

This is only used to invalidate page cache, so false positives are sort of acceptable since MVCC reads currently don’t read from WAL frames ever. FIXME: MVCC should start using pager read transactions anyway so that we can get rid of the stop-the-world MVCC checkpoint that blocks all reads.