pub struct Statement { /* private fields */ }Expand description
A prepared statement.
Implementations
Sourceimpl Statement
impl Statement
Sourcepub async fn query(&mut self, params: impl IntoParams) -> Result<Rows>
pub async fn query(&mut self, params: impl IntoParams) -> Result<Rows>
Query the database with this prepared statement.
Sourcepub async fn execute(&mut self, params: impl IntoParams) -> Result<u64>
pub async fn execute(&mut self, params: impl IntoParams) -> Result<u64>
Execute this prepared statement.
Sourcepub fn column_count(&self) -> usize
pub fn column_count(&self) -> usize
Returns the number of columns in the result set.
Sourcepub fn column_name(&self, idx: usize) -> Result<String>
pub fn column_name(&self, idx: usize) -> Result<String>
Returns the name of the column at the given index.
Sourcepub fn column_names(&self) -> Vec<String>
pub fn column_names(&self) -> Vec<String>
Returns the names of all columns in the result set.
Sourcepub fn column_index(&self, name: &str) -> Result<usize>
pub fn column_index(&self, name: &str) -> Result<usize>
Returns the index of the column with the given name.
Sourcepub fn reset(&self) -> Result<()>
pub fn reset(&self) -> Result<()>
Reset internal statement state after previous execution so it can be reused again