#[non_exhaustive]pub enum DropBehavior {
Rollback,
Commit,
Ignore,
Panic,
}Expand description
Options for how a Transaction should behave when it is dropped.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Rollback
Roll back the changes. This is the default.
Commit
Commit the changes.
Ignore
Do not commit or roll back changes - this will leave the transaction or savepoint open, so should be used with care.
Panic
Panic. Used to enforce intentional behavior during development.