ExtensionValue

Struct ExtensionValue 

pub struct ExtensionValue { /* private fields */ }

Implementations

Source

impl Value

pub const fn null() -> Value

Creates a new Value with type Null

pub const fn value_type(&self) -> ValueType

Returns the value type of the Value

pub fn to_float(&self) -> Option<f64>

Returns the float value or casts the relevant value to a float

pub fn to_text(&self) -> Option<&str>

Returns the text value if the Value is the proper type

pub fn to_text_coerced(&self) -> Option<String>

Returns the text value for any ValueType

pub fn is_json(&self) -> bool

pub fn to_blob(&self) -> Option<Vec<u8>>

Returns the blob value if the ValueType is Blob (copies the data)

pub fn blob_ref(&self) -> Option<&[u8]>

Returns a reference to the blob data without copying.

Unlike to_blob() which returns an owned Vec<u8>, this method returns a slice reference that borrows from the underlying storage.

pub fn to_integer(&self) -> Option<i64>

Returns the value as an integer (casts or converts if possible)

pub fn to_error(&self) -> Option<ResultCode>

Returns the error code if the value is an error

pub fn to_error_details(&self) -> Option<(ResultCode, Option<String>)>

Returns the error code and optional message if the value is an error

pub fn as_bytes(&self) -> Vec<u8> ⓘ

pub fn from_integer(i: i64) -> Value

Creates a new integer Value from an i64

pub fn from_float(value: f64) -> Value

Creates a new float Value from a f64

pub fn from_text(s: String) -> Value

Creates a new text Value from a String

pub fn from_json(s: String) -> Value

pub fn error(code: ResultCode) -> Value

Creates a new error Value from a ResultCode

pub fn error_with_message(message: String) -> Value

Creates a new error Value from a ResultCode and a message

pub fn from_blob(value: Vec<u8>) -> Value

Creates a new blob Value from a Vec

pub unsafe fn __free_internal_type(self)

Extension authors should not use this function, or enable the ‘core_only’ feature

Safety

consumes the value while freeing the underlying memory with null check. however this does assume that the type was properly constructed with the appropriate value_type and value.