TursoConnection

Struct TursoConnection 

pub struct TursoConnection { /* private fields */ }

Implementations

Source

impl TursoConnection

pub fn new( config: &TursoDatabaseConfig, connection: Arc<Connection>, ) -> Arc<Self>

pub fn new_with_sync_busy( config: &TursoDatabaseConfig, connection: Arc<Connection>, sync_busy: Option<Arc<SyncBusyGate>>, ) -> Arc<Self>

pub fn set_busy_timeout(&self, duration: Duration)

Set busy timeout for the connection

pub fn interrupt(&self)

Request interruption of the statement currently running on this connection. Mirrors sqlite3_interrupt: the in-flight step/execute aborts with an Interrupt error. Safe to call from another thread. If no statement is active the request is ignored.

pub fn set_query_timeout(&self, duration: Duration)

Set the maximum wall-clock duration a single statement is allowed to run before it is interrupted. Duration::ZERO disables the timeout.

pub fn get_query_timeout(&self) -> Duration

Get the current per-statement query timeout (Duration::ZERO when disabled).

pub fn get_auto_commit(&self) -> bool

pub fn last_insert_rowid(&self) -> i64

pub fn register_external_scalar_function( &self, name: String, argc: i32, deterministic: bool, context: usize, callback: ScalarFunction, context_destructor: Option<ContextDestructor>, value_destructor: Option<ValueDestructor>, ) -> Result<(), TursoError>

pub fn register_external_aggregate_function( &self, name: String, argc: i32, context: usize, init: InitAggFunction, step: StepFunction, finalize: FinalizeFunction, context_destructor: Option<ContextDestructor>, aggregate_destructor: Option<ContextDestructor>, value_destructor: Option<ValueDestructor>, ) -> Result<(), TursoError>

pub fn unregister_external_function(&self, name: &str) -> Result<(), TursoError>

pub fn register_external_collation( &self, name: String, context: usize, callback: ContextCollationFunction, context_destructor: Option<ContextDestructor>, )

pub fn unregister_external_collation(&self, name: &str)

pub fn set_load_extension_enabled(&self, enabled: bool)

pub fn load_extension(&self, path: &str) -> Result<(), TursoError>

pub fn prepare_single( &self, sql: impl AsRef<str>, ) -> Result<Box<TursoStatement>, TursoError>

prepares single SQL statement

pub fn prepare_cached( &self, sql: impl AsRef<str>, ) -> Result<Box<TursoStatement>, TursoError>

Prepare a statement from the provided SQL string and cache it for future use.

pub fn prepare_first( &self, sql: impl AsRef<str>, ) -> Result<Option<(Box<TursoStatement>, usize)>, TursoError>

prepares first SQL statement from the string and return prepared statement and position after the end of the parsed statement this method can be useful if SDK provides an execute(…) method which run all statements from the provided input in sequence

pub fn close(&self) -> Result<(), TursoError>

close the connection preventing any further operations executed over it SAFETY: caller must guarantee that no ongoing operations are running over connection before calling close(…) method

pub fn cacheflush(&self) -> Result<(), TursoError>

low-level method used only by the Rust SDK

pub fn to_capi(self: Arc<Self>) -> *mut turso_connection_t

helper method to get C raw container to the TursoConnection instance this method is used in the capi wrappers

pub unsafe fn ref_from_capi<'a>( value: *const turso_connection_t, ) -> Result<&'a Self, TursoError>

helper method to restore TursoConnection ref from C raw container this method is used in the capi wrappers

Safety

value must be a pointer returned from Self::to_capi method

pub unsafe fn arc_from_capi(value: *const turso_connection_t) -> Arc<Self>

helper method to restore TursoConnection instance from C raw container this method is used in the capi wrappers

Safety

value must be a pointer returned from Self::to_capi method