TursoDatabase

Struct TursoDatabase 

pub struct TursoDatabase { /* private fields */ }

Implementations

Source

impl TursoDatabase

pub const fn version() -> &'static str

return turso version

pub fn db_core(&self) -> Result<Arc<Database>, TursoError>

method to get turso_core::Database instance which can be useful for code which integrates with sdk-kit

pub fn io(&self) -> Result<Arc<dyn IO>, TursoError>

method to get turso_core::IO instance which can be useful for code which integrates with sdk-kit

pub fn new(config: TursoDatabaseConfig) -> Arc<Self>

create database holder struct but do not initialize it yet this can be useful for some environments, where IO operations must be executed in certain fashion (and open do IO under the hood)

pub fn open(&self) -> Result<IOResult<()>, TursoError>

Async version of database opening that returns IOResult. Caller must drive the IO loop and pass state between calls. This is useful for environments where IO operations must be executed in a specific fashion.

pub fn connect(&self) -> Result<Arc<TursoConnection>, TursoError>

creates database connection database must be already opened with Self::open method

pub fn to_capi(self: Arc<Self>) -> *mut turso_database_t

helper method to get C raw container with TursoDatabase instance this method is used in the capi wrappers

pub unsafe fn ref_from_capi<'a>( value: *const turso_database_t, ) -> Result<&'a Self, TursoError>

helper method to restore TursoDatabase ref from C raw container this method is used in the capi wrappers

Safety

value must be a pointer returned from Self::to_capi method

pub unsafe fn arc_from_capi(value: *const turso_database_t) -> Arc<Self>

helper method to restore TursoDatabase instance from C raw container this method is used in the capi wrappers

Safety

value must be a pointer returned from Self::to_capi method