Structs
- Database
Change - External
AggState - IOCompletion
Async - Immutable
Record - This struct serves the purpose of not allocating multiple vectors of bytes if not needed. A value in a record that has already been serialized can stay serialized and what this struct offsers is easy acces to each value which point to the payload. The name might be contradictory as it is immutable in the sense that you cannot modify the values without modifying the payload.
- Immutable
Record Ref - Index
Info - Metadata about an index, used for handling and comparing index keys.
- KeyInfo
- Record
- Serial
Type - Sqlite Serial Types https://www.sqlite.org/fileformat.html#record_format
- SumAgg
State - Text
- TextRef
- Value
Iterator - A zero-allocation iterator over SQLite record payload data.
- WalFrame
Info - WalState
Enums
- AggContext
- Aggregate context for accumulating values during GROUP BY. Built-in aggregates use a flat payload representation for efficiency and to share code between register-based and hash-based aggregation (future enhancement).
- Cursor
- Database
Change Type - IOCompletions
- IOResult
- Record
Compare - SeekKey
- SeekOp
- The match condition of a table/index seek.
- Seek
Result - Serial
Type Kind - Text
Subtype - Value
- Value
Ref - Value
Type - SQLite by default uses 2000 as maximum numbers in a row. It controlld by the constant called SQLITE_MAX_COLUMN But the hard limit of number of columns is 32,767 columns i16::MAX const MAX_COLUMN: usize = 2000;
Traits
- AnyBlob
- AnyText
- AsValue
Ref - Extendable
- From
Value - Convert a
Valueinto the implementors type.
Functions
- compare_
immutable - compare_
immutable_ iter - compare_
immutable_ single - compare_
records_ generic - Compare two table rows or index records.
- find_
compare - get_
serial_ type_ size - get_
tie_ breaker_ from_ seek_ op