pub enum AggContext {
Builtin(Vec<Value>),
External(ExternalAggState),
}Expand description
Aggregate context for accumulating values during GROUP BY. Built-in aggregates use a flat payload representation for efficiency and to share code between register-based and hash-based aggregation (future enhancement).
Variants
Builtin(Vec<Value>)
Built-in aggregates store state as a flat Vec
External(ExternalAggState)
External (extension) aggregates need FFI state that can’t be serialized.
Implementations
Sourceimpl AggContext
impl AggContext
pub fn compute_external(&self) -> Result<Value>
Sourcepub fn payload_mut(&mut self) -> &mut [Value]
pub fn payload_mut(&mut self) -> &mut [Value]
Get a mutable reference to the builtin payload as a slice
Sourcepub fn payload_vec_mut(&mut self) -> &mut Vec<Value>
pub fn payload_vec_mut(&mut self) -> &mut Vec<Value>
Get a mutable reference to the builtin payload Vec (for aggregates that grow the payload, e.g. array_agg).