pub enum SeekOp {
GE {
eq_only: bool,
},
GT,
LE {
eq_only: bool,
},
LT,
}Expand description
The match condition of a table/index seek.
Variants
GE
If eq_only is true, this means in practice: We are iterating forwards, but we are really looking for an exact match on the seek key.
GT
LE
If eq_only is true, this means in practice: We are iterating backwards, but we are really looking for an exact match on the seek key.
LT
Implementations
Sourceimpl SeekOp
impl SeekOp
Sourcepub fn iteration_direction(&self) -> IterationDirection
pub fn iteration_direction(&self) -> IterationDirection
A given seek op implies an iteration direction.
For example, a seek with SeekOp::GT implies: Find the first table/index key that compares greater than the seek key -> used in forwards iteration.
A seek with SeekOp::LE implies: Find the last table/index key that compares less than or equal to the seek key -> used in backwards iteration.
pub fn eq_only(&self) -> bool
pub fn reverse(&self) -> Self