pub struct IndexInfo {
pub key_info: IndexKeyInfo,
pub has_rowid: bool,
pub num_cols: usize,
pub is_unique: bool,
}Expand description
Metadata about an index, used for handling and comparing index keys.
This struct provides information about the sorting order of columns, whether the index includes a row ID, and the total number of columns in the index.
Fields
key_info: IndexKeyInfoSpecifies the sorting order (ascending or descending) for each column in the index.
has_rowid: boolIndicates whether the index includes a row ID column.
num_cols: usizeThe total number of columns in the index, including the row ID column if present.
is_unique: boolIndicates whether index rows should be unique.
Implementations
Sourceimpl IndexInfo
impl IndexInfo
pub fn key_info_in<A: ConcurrentAllocator>(alloc: A) -> IndexKeyInfo
pub fn key_info_from_iter_in<A, I>( key_info: I, _alloc: A, ) -> Result<IndexKeyInfo, TryReserveError>
pub fn new<I>(
key_info: I,
has_rowid: bool,
num_cols: usize,
is_unique: bool,
) -> Result<Self, TryReserveError>where
I: IntoIterator<Item = KeyInfo>,
pub fn new_in<A, I>( key_info: I, has_rowid: bool, num_cols: usize, is_unique: bool, alloc: A, ) -> Result<Self, TryReserveError>
pub fn new_from_index(index: &Index) -> Result<Self, TryReserveError>
pub fn new_from_index_in<A: ConcurrentAllocator>( index: &Index, alloc: A, ) -> Result<Self, TryReserveError>