pub enum CheckpointMode {
Passive {
upper_bound_inclusive: Option<u64>,
},
Full,
Restart,
Truncate {
upper_bound_inclusive: Option<u64>,
},
}Variants
Passive
Checkpoint as many frames as possible without waiting for any database readers or writers to finish, then sync the database file if all frames in the log were checkpointed. Passive never blocks readers or writers, only ensures (like all modes do) that there are no other checkpointers.
Optional upper_bound_inclusive parameter can be set in order to checkpoint frames with number no larger than the parameter
Full
This mode blocks until there is no database writer and all readers are reading from the most recent database snapshot. It then checkpoints all frames in the log file and syncs the database file. This mode blocks new database writers while it is pending, but new database readers are allowed to continue unimpeded.
Restart
This mode works the same way as Full with the addition that after checkpointing the log file it blocks (calls the busy-handler callback) until all readers are reading from the database file only. This ensures that the next writer will restart the log file from the beginning. Like Full, this mode blocks new database writer attempts while it is pending, but does not impede readers.
Truncate
This mode works the same way as Restart with the addition that it also truncates the log file to zero bytes just prior to a successful return.
Extra parameter can be set in order to perform conditional TRUNCATE: database will be checkpointed and truncated only if max_frames equals to the parameter value this behaviour used by sync-engine which consolidate WAL before checkpoint and needs to be sure that no frames will be missed