Modules
Structs
- Index
Method Configuration - Index
Method Cost Estimate - Cost estimate returned by custom index methods for optimizer integration. This enables the optimizer to make cost-based decisions when choosing between custom index methods and traditional BTree indexes.
- Index
Method Definition
Constants
Traits
- Index
Method - index method “entry point” which can create attachment of the method to the table with given configuration (this trait acts like a “factory”)
- Index
Method Attachment - index method attached to the table with specific configuration the attachment is capable of generating SELECT patterns where index can be used and also can create cursor for query execution
- Index
Method Cursor - cursor opened for index method and capable of executing DML/DDL/DQL queries for the index method over fixed table