pub struct IndexMethodDefinition<'a> {
pub method_name: &'a str,
pub index_name: &'a str,
pub patterns: &'a [Select],
pub backing_btree: bool,
pub results_materialized: bool,
}Fields
method_name: &'a strindex method name
index_name: &'a strindex name
patterns: &'a [Select]SELECT patterns where index method can be used the patterns can contain positional placeholder which will make planner to capture parameters from the original query and provide them to the index method (for example, pattern ‘SELECT * FROM {table} LIMIT ?’ will capture LIMIT parameter and provide its value from the query to the index method query_start(…) call)
backing_btree: boolspecial marker which forces tursodb core to treat index method as backing btree - so it will allocate real btree on disk for that index method
results_materialized: boolWhether query_start() materializes all matching rowids up front (e.g. into a Vec/VecDeque).
When true, the cursor is safe to use during DML because it does not lazily stream from
a live data structure that writes could invalidate.
When false, the emitter will collect rowids into a RowSet/ephemeral table before writing.