pub trait IndexMethod:
Debug
+ Send
+ Sync {
// Required method
fn attach(
&self,
configuration: &IndexMethodConfiguration,
) -> Result<Arc<dyn IndexMethodAttachment>>;
}Expand description
index method “entry point” which can create attachment of the method to the table with given configuration (this trait acts like a “factory”)
Required Methods
Sourcefn attach(
&self,
configuration: &IndexMethodConfiguration,
) -> Result<Arc<dyn IndexMethodAttachment>>
fn attach( &self, configuration: &IndexMethodConfiguration, ) -> Result<Arc<dyn IndexMethodAttachment>>
create attachment of the index method to the specific table with specific method configuration
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".