pub trait IndexMethodAttachment:
Debug
+ Send
+ Sync {
// Required methods
fn definition<'a>(&'a self) -> IndexMethodDefinition<'a>;
fn init(&self) -> Result<Box<dyn IndexMethodCursor>>;
}Expand description
index method attached to the table with specific configuration the attachment is capable of generating SELECT patterns where index can be used and also can create cursor for query execution
Required Methods
fn definition<'a>(&'a self) -> IndexMethodDefinition<'a>
fn init(&self) -> Result<Box<dyn IndexMethodCursor>>
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".