pub struct RefEntry<'a, K, V, C = BasicComparator, A: SkiplistAllocator = TursoAllocator> { /* private fields */ }Expand description
A reference-counted entry in a skip list.
You must call release to free this type, otherwise the node will be
leaked. This is because releasing the entry requires a Guard.
Implementations
Sourceimpl<'a, K: 'a, V: 'a, C: 'a, A: SkiplistAllocator> RefEntry<'a, K, V, C, A>
impl<'a, K: 'a, V: 'a, C: 'a, A: SkiplistAllocator> RefEntry<'a, K, V, C, A>
Sourcepub fn is_removed(&self) -> bool
pub fn is_removed(&self) -> bool
Returns true if the entry is removed from the skip list.
Sourcepub fn release_with_pin<F>(self, pin: F)
pub fn release_with_pin<F>(self, pin: F)
Releases the reference of the entry, pinning the thread only when the reference count of the node becomes 0.
Sourceimpl<K, V, C, A: SkiplistAllocator> RefEntry<'_, K, V, C, A>
impl<K, V, C, A: SkiplistAllocator> RefEntry<'_, K, V, C, A>
Sourceimpl<K, V, C, A: SkiplistAllocator> RefEntry<'_, K, V, C, A>where
C: Comparator<K>,
impl<K, V, C, A: SkiplistAllocator> RefEntry<'_, K, V, C, A>where
C: Comparator<K>,