pub struct TursoDatabaseConfig {
pub path: String,
pub experimental_features: Option<String>,
pub async_io: bool,
pub encryption: Option<EncryptionOpts>,
pub vfs: Option<String>,
pub io: Option<Arc<dyn IO>>,
pub db_file: Option<Arc<dyn DatabaseStorage>>,
}Fields
path: Stringpath to the database file or “:memory:” for in-memory connection
experimental_features: Option<String>comma-separated list of experimental features to enable this field is intentionally just a string in order to make enablement of experimental features as flexible as possible
async_io: boolif true, library methods will return Io status code and delegate Io loop to the caller if false, library will spin IO itself in case of Io status code and never return it to the caller
encryption: Option<EncryptionOpts>optional encryption parameters for local data encryption as encryption is experimental - Self::experimental_features must have “encryption” in the list
vfs: Option<String>optional VFS parameter explicitly specifying FS backend for the database. Available options are:
- “memory”: in-memory backend
- “syscall”: generic syscall backend
- “io_uring”: IO uring (supported only on Linux)
io: Option<Arc<dyn IO>>optional custom IO provided by the caller
db_file: Option<Arc<dyn DatabaseStorage>>optional custom DatabaseStorage provided by the caller if provided, caller must guarantee that IO used by the TursoDatabase will be consistent with underlying DatabaseStorage IO
Implementations
Sourceimpl TursoDatabaseConfig
impl TursoDatabaseConfig
Sourcepub fn database_opts(&self) -> DatabaseOpts
pub fn database_opts(&self) -> DatabaseOpts
Build the typed turso_core::DatabaseOpts from the comma-separated
Self::experimental_features string. The feature-name tokens are the
SDK/CLI-facing names; unknown names are ignored and "strict" is a
no-op (strict tables are always enabled). This keeps the
string -> typed-options translation in the SDK layer where the string
representation lives, instead of in turso_core.
Sourceimpl TursoDatabaseConfig
impl TursoDatabaseConfig
Sourcepub unsafe fn from_capi(
config: *const turso_database_config_t,
) -> Result<Self, TursoError>
pub unsafe fn from_capi( config: *const turso_database_config_t, ) -> Result<Self, TursoError>
helper method to restore TursoSetupConfig instance from C representation this method is used in the capi wrappers
Safety
c::turso_database_config_t::path field must be valid C-string pointer c::turso_database_config_t::experimental_features field must be valid C-string pointer or null