Module types

Module types 

Structs

DatabaseChange
ExternalAggState
IOCompletionAsync
ImmutableRecord
This struct serves the purpose of not allocating multiple vectors of bytes if not needed. A value in a record that has already been serialized can stay serialized and what this struct offsers is easy acces to each value which point to the payload. The name might be contradictory as it is immutable in the sense that you cannot modify the values without modifying the payload.
ImmutableRecordRef
IndexInfo
Metadata about an index, used for handling and comparing index keys.
KeyInfo
Record
SerialType
Sqlite Serial Types https://www.sqlite.org/fileformat.html#record_format
SumAggState
Text
TextRef
ValueIterator
A zero-allocation iterator over SQLite record payload data.
WalFrameInfo
WalState

Enums

AggContext
Aggregate context for accumulating values during GROUP BY. Built-in aggregates use a flat payload representation for efficiency and to share code between register-based and hash-based aggregation (future enhancement).
Cursor
DatabaseChangeType
IOCompletions
IOResult
RecordCompare
SeekKey
SeekOp
The match condition of a table/index seek.
SeekResult
SerialTypeKind
TextSubtype
Value
ValueRef
ValueType
SQLite by default uses 2000 as maximum numbers in a row. It controlld by the constant called SQLITE_MAX_COLUMN But the hard limit of number of columns is 32,767 columns i16::MAX const MAX_COLUMN: usize = 2000;

Traits

AnyBlob
AnyText
AsValueRef
Extendable
FromValue
Convert a Value into the implementors type.

Functions

compare_immutable
compare_immutable_iter
compare_immutable_single
compare_records_generic
Compare two table rows or index records.
find_compare
get_serial_type_size
get_tie_breaker_from_seek_op

Type Aliases

IndexKeyInfo